@@ -1,6 +1,6 @@
 package Lemonldap::NG::Handler::Main::Reload;
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.0.12';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -59,9 +59,16 @@ sub checkConf {
     }
     $Lemonldap::NG::Common::Conf::msg = '';
 
-    if ( $force or !$class->cfgNum or $class->cfgNum != $conf->{cfgNum} ) {
+    if (   $force
+        or !$class->cfgNum
+        or !$class->cfgDate
+        or $class->cfgNum != $conf->{cfgNum}
+        or $class->cfgDate != $conf->{cfgDate} )
+    {
         $class->logger->debug("Get configuration $conf->{cfgNum}");
-        unless ( $class->cfgNum( $conf->{cfgNum} ) ) {
+        unless ( $class->cfgNum( $conf->{cfgNum} )
+            && $class->cfgDate( $conf->{cfgDate} ) )
+        {
             $class->logger->error('No configuration available');
             return 0;
         }
@@ -80,7 +87,7 @@ sub checkConf {
             }
         }
     }
-    $class->tsv->{checkTime} = $conf->{checkTime} if ( $conf->{checkTime} );
+    $class->checkTime( $conf->{checkTime} ) if $conf->{checkTime};
     $class->lastCheck( time() );
     $class->logger->debug("$class: configuration is up to date");
     return 1;
@@ -166,7 +173,7 @@ sub configReload {
 
     foreach my $sub (
         qw( defaultValuesInit jailInit portalInit locationRulesInit
-        sessionStorageInit headersInit postUrlInit aliasInit )
+        sessionStorageInit headersInit postUrlInit aliasInit oauth2Init )
       )
     {
         $class->logger->debug("Process $$ calls $sub");
@@ -203,7 +210,8 @@ sub defaultValuesInit {
         timeoutActivityInterval useRedirectOnError useRedirectOnForbidden
         useSafeJail             whatToTrace        handlerInternalCache
         handlerServiceTokenTTL  customToTrace      lwpOpts lwpSslOpts
-        authChoiceParam         authChoiceAuthBasic
+        authChoiceAuthBasic     authChoiceParam    hiddenAttributes
+        upgradeSession
         )
     );
 
@@ -610,6 +618,25 @@ sub aliasInit {
 
 # TODO: support wildcards in aliases
 
+## @method arrayref oauth2Init
+# @param options vhostOptions configuration item
+# @return arrayref of vhost and aliases
+sub oauth2Init {
+    my ( $class, $conf ) = @_;
+
+    foreach my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} || {} } ) {
+        if (    $conf->{oidcRPMetaDataOptions}->{$rp}
+            and $conf->{oidcRPMetaDataOptions}->{$rp}
+            ->{oidcRPMetaDataOptionsClientID} )
+        {
+            $class->tsv->{oauth2Options}->{$rp}->{clientId} =
+              $conf->{oidcRPMetaDataOptions}->{$rp}
+              ->{oidcRPMetaDataOptionsClientID};
+        }
+    }
+    return 1;
+}
+
 sub substitute {
     my ( $class, $expr ) = @_;
 
