@@ -1,7 +1,7 @@
 # Main running methods file
 package Lemonldap::NG::Handler::Main::Run;
 
-our $VERSION = '2.0.9';
+our $VERSION = '2.0.12';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -105,6 +105,7 @@ sub checkType {
 sub run {
     my ( $class, $req, $rule, $protection ) = @_;
     my ( $id, $session );
+    my $vhost = $class->resolveAlias($req);
 
     return $class->DECLINED unless ( $class->is_initial_req($req) );
 
@@ -138,7 +139,9 @@ sub run {
     }
 
     # Try to recover cookie and user session
-    if (    $id = $class->fetchId($req)
+    $id = $class->fetchId($req);
+    $class->data( {} ) unless($id);
+    if (    $id
         and $session = $class->retrieveSession( $req, $id ) )
     {
 
@@ -149,9 +152,41 @@ sub run {
 
         # ACCOUNTING (1. Inform web server)
         $class->set_user( $req, $session->{ $class->tsv->{whatToTrace} } );
-        $class->set_custom( $req, $session->{ $class->tsv->{customToTrace} } )
-          if $class->tsv->{customToTrace}
-          and $session->{ $class->tsv->{customToTrace} };
+
+        my $custom;
+        $custom = $session->{ $class->tsv->{customToTrace} }
+          if (  $class->tsv->{customToTrace}
+            and $session->{ $class->tsv->{customToTrace} } );
+        if ( $class->tsv->{accessToTrace}->{$vhost} ) {
+            my ( $function, @params ) = split /\s*,\s*/,
+              $class->tsv->{accessToTrace}->{$vhost};
+            if ( $function =~ qr/^(?:\w+(?:::\w+)*(?:\s+\w+(?:::\w+)*)*)?$/ ) {
+                my $c = eval {
+                    no strict 'refs';
+                    &{$function}( {
+                            req     => $req,
+                            vhost   => $vhost,
+                            session => $session,
+                            custom  => $custom,
+                            params  => \@params
+                        }
+                    );
+                };
+                if ($@) {
+                    $class->logger->error(
+                        "Failed to overwrite customToTrace: $@");
+                }
+                else {
+                    $class->logger->debug("Overwrite customToTrace with: $c");
+                    $custom = $c;
+                }
+            }
+            else {
+                $class->logger->error(
+                    "accessToTrace: Bad custom function name");
+            }
+        }
+        $class->set_custom( $req, $custom ) if $custom;
 
         # AUTHORIZATION
         return ( $class->forbidden( $req, $session ), $session )
@@ -195,6 +230,10 @@ sub run {
     elsif ( $protection == $class->MAYSKIP
         and $class->grant( $req, $session, $uri, $cond ) eq '999_SKIP' )
     {
+        $class->logger->debug("Access control skipped");
+        $class->updateStatus( $req, 'SKIP' );
+        $class->hideCookie($req);
+        $class->cleanHeaders($req);
         return $class->OK;
     }
 
@@ -309,7 +348,6 @@ sub grant {
     return $cond->( $req, $session ) if ($cond);
 
     $vhost ||= $class->resolveAlias($req);
-
     my $level = $class->getLevel( $req, $uri );
 
     # Using VH authentification level if exists
@@ -319,9 +357,11 @@ sub grant {
                 "User authentication level = $session->{authenticationLevel}");
             $class->logger->debug("Required authentication level = $level");
             $class->logger->warn(
-"User rejected due to insufficient authentication level -> Session upgrade enabled"
-            );
-            $session->{_upgrade} = 1;
+                'User rejected due to insufficient authentication level');
+            if ( $class->tsv->{upgradeSession} ) {
+                $class->logger->warn(' -> Session upgrade enabled');
+                $session->{_upgrade} = 1;
+            }
             return 0;
         }
     }
@@ -465,10 +505,9 @@ sub fetchId {
     my $lookForHttpCookie = ( $class->tsv->{securedCookie} =~ /^(2|3)$/
           and not $class->_isHttps( $req, $vhost ) );
     my $cn = $class->tsv->{cookieName};
-    my $value =
-      $lookForHttpCookie
-      ? ( $t =~ /${cn}http=([^,; ]+)/o ? $1 : 0 )
-      : ( $t =~ /$cn=([^,; ]+)/o       ? $1 : 0 );
+    my $value = $lookForHttpCookie    # Avoid prefix and bad cookie name (#2417)
+      ? ( $t =~ /(?<![-.~])\b${cn}http=([^,; ]+)/o ? $1 : 0 )
+      : ( $t =~ /(?<![-.~])\b$cn=([^,; ]+)/o       ? $1 : 0 );
 
     if ( $value && $lookForHttpCookie && $class->tsv->{securedCookie} == 3 ) {
         $value = $class->tsv->{cipher}->decryptHex( $value, "http" );
@@ -613,7 +652,7 @@ sub _getPort {
             return $class->tsv->{port}->{_};
         }
         else {
-            return $req->{env}->{SERVER_PORT};
+            return $req->port;
         }
     }
 }
@@ -637,7 +676,7 @@ sub _isHttps {
             return $class->tsv->{https}->{_};
         }
         else {
-            return ( uc( $req->{env}->{HTTPS} || "OFF" ) eq "ON" );
+            return $req->secure;
         }
     }
 }
@@ -732,6 +771,7 @@ sub cleanHeaders {
     my ( $class, $req ) = @_;
     my $vhost = $class->resolveAlias($req);
     if ( defined( $class->tsv->{headerList}->{$vhost} ) ) {
+        $class->logger->debug("Remove headers relative to $vhost");
         $class->unset_header_in( $req,
             @{ $class->tsv->{headerList}->{$vhost} } );
     }
@@ -793,10 +833,14 @@ sub localUnlog {
     if ( $id //= $class->fetchId($req) ) {
 
         # Delete local cache
-        if (    $class->tsv->{refLocalStorage}
-            and $class->tsv->{refLocalStorage}->get($id) )
-        {
-            $class->tsv->{refLocalStorage}->remove($id);
+        if ( $class->tsv->{sessionCacheModule} ) {
+            my $module  = $class->tsv->{sessionCacheModule};
+            my $options = $class->tsv->{sessionCacheOptions};
+            eval "use $module;";
+            my $cache = $module->new($options);
+            if ( $cache->get($id) ) {
+                $cache->remove($id);
+            }
         }
     }
 }
